'use strict';

exports.__esModule = true;
exports.default = devTools;

var _jsan = require('jsan');

var _socketclusterClient = require('socketcluster-client');

var _socketclusterClient2 = _interopRequireDefault(_socketclusterClient);

var _configureStore = require('./configureStore');

var _configureStore2 = _interopRequireDefault(_configureStore);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var socket = undefined;
var channel = undefined;
var store = {};
var shouldInit = true;
var actionsCount = 0;
var lastTime = 0;

function relay(type, state, action, nextActionId) {
  var message = {
    payload: state ? (0, _jsan.stringify)(state) : '',
    action: action ? (0, _jsan.stringify)(action) : '',
    nextActionId: nextActionId || '',
    type: type,
    init: shouldInit
  };
  if (shouldInit) shouldInit = false;

  socket.emit('log', message);
}

function handleMessages(message) {
  if (message.type === 'DISPATCH') {
    store.liftedStore.dispatch(message.action);
  } else if (message.type === 'UPDATE') {
    relay('STATE', store.liftedStore.getState());
  }
}

function init() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? _constants.socketOptions : arguments[0];

  socket = _socketclusterClient2.default.connect(options);

  socket.emit('login', 'master', function (err, channelName) {
    if (err) {
      console.error(err);return;
    }
    channel = socket.subscribe(channelName);
    channel.watch(handleMessages);
  });
}

function subscriber() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var action = arguments[1];

  if (action && action.type) {
    setTimeout(function () {
      if (action.type === 'PERFORM_ACTION') {
        if (lastTime > action.timestamp) return state;
        actionsCount++;
        relay('ACTION', store.getState(), action, actionsCount);
      } else {
        var liftedState = store.liftedStore.getState();
        lastTime = Date.now();
        relay('STATE', liftedState);
        actionsCount = liftedState.nextActionId;
      }
    }, 0);
  }
  return state;
}

function devTools(options) {
  return function (next) {
    return function (reducer, initialState) {
      init(options);
      store = (0, _configureStore2.default)(next, subscriber)(reducer, initialState);
      return store;
    };
  };
}